// DON'T CHANGE THIS FILE MANUALLY
// This file was generated using the command:
// $ goi18n constants

package R

// DDays is the identifier for the following localizable string template(s):
// one: "{{.Count}} day"
// other: "{{.Count}} days"
const DDays = "d_days"

// MyHeightInMeters is the identifier for the following localizable string template(s):
// one: "I am {{.Count}} meter tall."
// other: "I am {{.Count}} meters tall."
const MyHeightInMeters = "my_height_in_meters"

// PersonGreeting is the identifier for the following localizable string template(s):
// "Hello {{.Person}}"
const PersonGreeting = "person_greeting"

// PersonUnreadEmailCount is the identifier for the following localizable string template(s):
// one: "{{.Person}} has {{.Count}} unread email."
// other: "{{.Person}} has {{.Count}} unread emails."
const PersonUnreadEmailCount = "person_unread_email_count"

// PersonUnreadEmailCountTimeframe is the identifier for the following localizable string template(s):
// one: "{{.Person}} has {{.Count}} unread email in the past {{.Timeframe}}."
// other: "{{.Person}} has {{.Count}} unread emails in the past {{.Timeframe}}."
const PersonUnreadEmailCountTimeframe = "person_unread_email_count_timeframe"

// ProgramGreeting is the identifier for the following localizable string template(s):
// "Hello world"
const ProgramGreeting = "program_greeting"

// YourUnreadEmailCount is the identifier for the following localizable string template(s):
// one: "You have {{.Count}} unread email."
// other: "You have {{.Count}} unread emails."
const YourUnreadEmailCount = "your_unread_email_count"
